;;
;; COMPONENT ARRAY DATA STRUCTURE
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Generate data structure and array of type-T components
;;      Macro to generate all the data structure required
;;      to manage an array of components of the same type.
;;  It generates these labels:
;;      * T_num:    A byte to count the number of elements in the array
;;      * T_pend:   A word to store a pointer to the end of the array.
;;                  This will be the first free element in the array.
;;      * T_array:  The array itself.
;;  INPUT:
;;      Tname:  Name of the component type
;;      N:      Size of the array in number of components (to reserve size)
;;      DefineTypeMacroDefault: Macro to be called to generate a default entity

.macro DefineComponentArrayStructure _Tname, _N, _DefineTypeMacroDefault
    _Tname'_num:        .db 0                   ;; Number of defined elements
    _Tname'_pend:       .dw _Tname'_array       ;; Pointer to the end of the array
    _Tname'_array:                              ;; Array of elements of the default type
    .rept   _N
        _DefineTypeMacroDefault                 ;; Repeatedly call default call 
    .endm
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Same macro as before but generating a bunch of zeroes instead of default values
;;  It uses _ComponentSize instead of _DefineTypeMacroDefault to produce the array
;;  INPUT:
;;      Same as before except
;;      _ComponentSize: Size in bytes of one T-component

.macro DefineComponentArrayStructure_Size _Tname, _N, _ComponentSize
    _Tname'_num:        .db 0               ;; Number of define elements
    _Tname'_pend:       .dw _Tname'_array   ;; Pointer to the end of the array
    _Tname'_array:                          ;; Array of elements of the desired type
      .ds   _N * _ComponentSize             ;; Produce as many 0's as ComponentSize
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Generate data structure and array of pointers 
;;      Macro to generate all the data structure required
;;      to manage an array of pointers.
;;  It generates these labels:
;;      _Tname_ptr_pend:    Pointer to the last element of the array
;;      _Tname_ptr_array:   Pointer to the beginning of the array
;;  INPUT:
;;      Tname:  Name of the component type
;;      N:      Size of the array in number of components
;;
.macro DefineComponentPointersArrayStructure_Size _Tname, _N
    _Tname'_ptr_pend:       .dw _Tname'_ptr_array
    _Tname'_ptr_array:
    .rept   _N
        .dw     0x0000
    .endm
.endm


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Same macro as before, but unnamed
.macro DefineEmptyComponentPointersArrayStructure_Size _N
    ;; Pend
    .dw 0x0000 + . + 2
    ;; Data
    .rept   _N
        .dw     0x0000
    .endm
.endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Generate component pointers array 
;;      Macro to generate all the data structure required
;;      to manage entity components.
;;  It generates these labels:
;;      _PTname_access_table: Start of the component pointers
;;      _PTname_components: Pointer to the beginning of the components array structure
;;  INPUT:
;;      _PTname:  Name of the pointers table
;;      _Num_Components: Number of array components to store
;;      _Num_Entities: Size of each component array
;;
__component_iteration = 0
.macro DefineComponentPointersTable _PTname, _Num_Components, _Num_Entities
    ;; Pointers of component pointers
    _PTname'_access_table:
    .rept   _Num_Components
        ;; Pointer to the beginning of each component array structure
        .dw    0x0000 + . + _Num_Components * 2 + (__component_iteration * ( _Num_Entities * 2 ) )
        __component_iteration = __component_iteration + 1
    .endm
    ;; Create a component pointer array for each component
    _PTname'_components:
    .rept   _Num_Components
        DefineEmptyComponentPointersArrayStructure_Size _Num_Entities
    .endm
.endm