;;
;;  ENTITY COMPONENT
;;  ================================================================================
;;    ----------- Component Type ---------------------------------------------------
;;    1B Component-Type         (cmp_type)
;;
;;    ----------- Position ---------------------------------------------------------
;;    2B Position               (x, y)
;;
;;    ----------- Velocity (Physics) -----------------------------------------------
;;    2B Velocity               (vx, vy)
;;
;;    ----------- Animation --------------------------------------------------------
;;    2B Animation Pointer      (anim_ptr)
;;	1B Animation Status	  (anim_st)
;;    ----------- Bounding Box -----------------------------------------------------
;;    2B Size                   (sx, sy) or (w, h)
;;
;;    ----------- Render -----------------------------------------------------------
;;    2B Sprite Pointer         (pspr)
;;    2B Last Sprite Pointer    (last_pspr)
;;    2B Last Video Pointer     (lastVP)
;;
;;    ----------- AI ---------------------------------------------------------------
;;    2B AI-AIM Target                   (ai_aim_x, ai_aim_y)
;;    1B AI-Status                       (ai_st)
;;    1B AI-Previous-Status              (ai_pre_st)
;;    2B AI-Next-Patrol-Step Pointer     (ai_patrol_step)
;;    2B AI-Next-Routine Pointer         (ai_next_routine)
;;
;;    ----------- Jump -------------------------------------------------------------
;;    1B Jump State      (jumpstate)
;;
;;    ----------- Melee ------------------------------------------------------------
;;    1B Melee Attack State              (meleeattackstate)
;;    2B Last Melee Attack Video Pointer (melee_lastVP)
;;    2B Melee Attack Size               (melee_attack_width, melee_attack_height)
;;
;;    ----------- Status -----------------------------------------------------------
;;    1B Orientation            (orientation)
;;	1B Previous Orientation	  (prev_orientation)
;;    1B Lifes counter          (lifes_counter)
;;    1B Colision Timeout       (colision_timeout)
;;    1B Status                 (status)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.include "sys/ai_control.h.s"   ;; sys_ai_stand_by

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Easily assign offsets to Entity_t members without error
;;  It adds sizes of types to an offset constant that assigns it
;;  to the next offset
__off = 0
.macro DefOffset    _size, _name
    _name   = __off             ;; Define name constant on curent offset
    __off   = __off + _size     ;; Add sizeof(_name) = _size to offset
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Easily define an enumeration starting in 0
;;  It adds sizes of types to an offset constant that assigns to
;;  Define enumeration name
.macro DefEnum  _name
    _name'_offset = 0
.endm
;;  Define enumeration element for an enumeration name
.macro Enum     _enumname, _element
    _enumname'_'_element = _enumname'_offset
    _enumname'_offset = _enumname'_offset + 1
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Easily define an animation step with a number of frames to 
;;  be completed before moving to next step
;;  Define animation step pointer name with its duration frames
.macro DefAnimStepWithFrames _animptr, _frames_remaining, _frames
    .dw _animptr
    .db _frames_remaining
    .db _frames
.endm
;; End of animation step, which must be a pointer to the beginning of the animation
.macro DefAnimStepEnd _animptr_start
    .dw nullptr
    .dw _animptr_start
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Default constructor for Entity_t
;;
.macro DefineCmp_Entity_default
    DefineCmp_Entity    0,  0,  0,  0,  e_w_invalidEntity,  1,  e_cmp_default, nullptr, nullptr, e_ai_st_noAI
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Defines a new Entity_t structure
;;  All Entity_t data together to simplify access, at the cost of memory space
;;
.macro DefineCmp_Entity _x, _y, _vx, _vy, _w, _h, _cmp_type, _pspr, _animptr, _aist
    .narg __argn
    .if __argn - 10  ;; Validate parameter number
        .error 1
    .else
        ;;;;; Component type
        .db     _cmp_type         ;; Types of components the entity has (bit-array)
        ;;;;; CMP: Position
        .db     _x,   _y          ;; Position
        ;;;;; CMP: Velocity
        .db     _vx,  _vy         ;; Velocity
        ;;;;; CMP: Animation
        .dw     _animptr          ;; Animation Pointer (anim_ptr)
        .db     0x00 	          ;; Animation Status  (anim_st)
        ;;;;; CMP: Bounding Box
        .db     _w,   _h          ;; Size
        ;;;;; CMP: Render
        .dw     _pspr             ;; Sprite Pointer
        .dw     nullptr           ;; Last Sprite Pointer
        .dw    0xCCCC             ;; Last Video Memory Pointer value (for faster clearing of the sprite)
        ;;;;; CMP: AI
        .db    0x00, 0x00         ;; AI-AIM    (ai_aim_x, ai_aim_y)
        .db    _aist              ;; AI-Status (ai_st)
        .db    _aist              ;; AI-Previous-Status  (ai_pre_st)
        .dw    nullptr            ;; Next AI Patrol Step (ai_patrol_step)
        .dw    sys_ai_stand_by    ;; Next AI Routine to call according to status (default to stand_by)
        ;;;;; CMP: Jump
        .db     jumpstate_nojump  ;; Jump state
                                  ;; Default to -1 to indicate the character isn't jumping
        ;;;;; CMP: Melee
        .db     #-1               ;; Melee Attack State (meleeattackstate)
        .dw     nullptr           ;; Last Melee Attack Video Pointer value (for faster clearing of the sprite)
        .db     #4, #5            ;; Melee Attack Size (width, height)
        ;;;;; CMP: Status
        .db     e_orientation_right ;; Orientation (looking right of left)
        .db     e_orientation_right ;; Previous Orientation
        .db     lifes_default     ;; Lifes counter
        .db     #0                ;; Colision Timeout (for invulnerability status)
        .db     e_st_standard     ;; Status - normally standard
        .db     e_gr_st_ground
    .endif
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Defines an array of Entity_t with default values
;;
.macro DefineCmpArray_Entity _N
    .rept _N
        DefineCmp_Entity_default
    .endm
.endm

;; -----------------------------
;; Entity_t offsets
;; -----------------------------
;; CMP: Component Type --------------------------------------------------------------
DefOffset   1, e_cmp_type

;; CMP: Position --------------------------------------------------------------------
DefOffset   1, e_x
DefOffset   1, e_y

;; CMP: Velocity (Physics) ----------------------------------------------------------
DefOffset   1, e_vx
DefOffset   1, e_vy

;; CMP: Animation -------------------------------------------------------------------
DefOffset   1, e_anim_ptr_l
DefOffset   1, e_anim_ptr_h
DefOffset   1, e_anim_st

;; CMP: Bounding Box ----------------------------------------------------------------
DefOffset   1, e_w
DefOffset   1, e_h

;; CMP: Render ----------------------------------------------------------------------
DefOffset   1, e_pspr_l
DefOffset   1, e_pspr_h

DefOffset   1, e_last_pspr_l  ;; / Save last sprite pointer (needed for animations)
DefOffset   1, e_last_pspr_h	;; \

DefOffset   1, e_lastVP_l
DefOffset   1, e_lastVP_h

;; CMP: AI --------------------------------------------------------------------------
DefOffset   1, e_ai_aim_x
DefOffset   1, e_ai_aim_y

DefOffset   1, e_ai_st
DefOffset   1, e_ai_pre_st

DefOffset   1, e_ai_patrol_step_l
DefOffset   1, e_ai_patrol_step_h

DefOffset   1, e_ai_next_routine_l
DefOffset   1, e_ai_next_routine_h

;; CMP: Jump ------------------------------------------------------------------------
DefOffset   1, e_jumpstate

;; CMP: Melee -----------------------------------------------------------------------
DefOffset   1, e_meleeattackstate

DefOffset   1, e_melee_lastVP_l
DefOffset   1, e_melee_lastVP_h

DefOffset   1, e_melee_attack_width
DefOffset   1, e_melee_attack_height

;; CMP: Status ----------------------------------------------------------------------
DefOffset   1, e_orientation            ;; if looking left or right
DefOffset   1, e_prev_orientation
DefOffset   1, e_lifes_counter
DefOffset   1, e_colision_timeout
DefOffset   1, e_status

;; CMP: Ground stats ----------------------------------------------------------------
DefOffset   1, e_gr_st

;; Size of entity, always last offset -----------------------------------------------
DefOffset   0, sizeof_e

;;--------------------------------
;; Entity_t AI-status enum
;;--------------------------------
DefEnum e_ai_st
Enum    e_ai_st noAI
Enum    e_ai_st stand_by
Enum    e_ai_st move_to
Enum    e_ai_st patrol

;;--------------------------------
;; Anim_t Status enum
;;--------------------------------
;; Anim status bit values:
;; UDRL
;; 0000
DefEnum e_anim_st
Enum    e_anim_st standing		;; 0000
Enum    e_anim_st walking_right	;; 0010
Enum    e_anim_st walking_left 	;; 0001
Enum    e_anim_st jumping_right	;; 1010
Enum    e_anim_st jumping_left 	;; 1001


;; Important jumpstates
jumpstate_nojump    = -1
jumpstate_end       = 0x80

;; Melee attack constants
;; Not attacking is != -1
melee_notattacking      = -1
melee_attackduration    = 20

;; lifes constants
lifes_default       = 3
lifes_no_lifes      = 0

;; Colision constants
colision_timeout_duration       = 20
colision_no_timeout_colision    = 0

;;----------------------------------
;; Entity_t Component Types
;;----------------------------------
e_cmp_AI        = 0x01                          ;; 0b00000001
e_cmp_Physics   = 0x02                          ;; 0b00000010
e_cmp_Animation = 0x04                          ;; 0b00000100
e_cmp_Render    = 0x08                          ;; 0b00001000
e_cmp_default   = e_cmp_Render | e_cmp_Physics

;;----------------------------------
;; Entity_t Component IDs
;;----------------------------------
DefEnum e_cmpID
Enum    e_cmpID AI              ;; 0
Enum    e_cmpID Physics         ;; 1
Enum    e_cmpID Animation       ;; 2
Enum    e_cmpID Num_Components  ;; 3

;;----------------------------------
;; Entity_t Status enum
;;----------------------------------
e_w_invalidEntity   = 0xFF  ;; Entity width = -1 means invalid entity

;;----------------------------------
;; Utility definitions
;;----------------------------------
nullptr         = 0x0000

;;----------------------------------
;; Entity Status definitions
;;----------------------------------
DefEnum e_st
Enum e_st standard
Enum e_st lostlife     ;; Only por PJ
Enum e_st dead         ;; Enemy recieved attack

;;----------------------------------
;; Entity_t status for  this
;;----------------------------------
DefEnum e_gr_st
Enum    e_gr_st  ground
Enum    e_gr_st  not_ground

;;----------------------------------
;; Entity Orientation
;;----------------------------------
e_orientation_left  = 0
e_orientation_right = 1

;; Jump Table
jumptable:
    .db     -5, -5, -4, -4
    .db     -3, -3, -3, -2
    .db     -2, -2, -1, -1
    .db     -1, 00, 00;, 01
;    .db     01, 01, 02, 02
;    .db     02, 03, 03, 04
    .db     0x80

new_jump:
    .db     -4, -3, -3, -2
    .db     -2, -2, -1, -1
    .db     -1, 00, 00, 01
    .db     01, 01, 02, 02
    .db     02, 03, 03, 04
    .db     0x80

;;----------------------------------
;; Utils enum
;;----------------------------------
true = 1
false = 0
