;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ENTITY MANAGER
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cpctelera.h.s"
.include "cmp/array_structure.h.s"
.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "man/components.h.s"
.include "assets/assets.h.s"

.module entity_manager

;; ==========================================================================
;; Manager Member Variables

;; ------------------------- Entity components ------------------------------
DefineComponentArrayStructure _entity,  max_entities, DefineCmp_Entity_default
DefineComponentArrayStructure _blocks,  num_blocks  , DefineCmp_Entity_default

;; ==========================================================================
;; ==========================================================================
;; Manager Public Functions

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man Entity::getArray
;; -------------------------------------------------------------
;;   Gets a pointer to the array of entities
;;   in IX and also the number of entities in A
;; INPUT: -
;; DESTROYS: A, IX
;; RETURNS:
;;     A: Number of entities in the array
;;     IX: Pointer to the start of the array
;;
man_entity_getArray::
    ld  ix, #_entity_array      ;; [4] IX = Pointer to the entitiy array
    ld   a, (_entity_num)       ;; [4]  A = Number of entities
    ret                         ;; [3]

man_entity_getArray_IY::
    ld  iy, #_entity_array      ;; [4] IX = Pointer to the entitiy array
    ld   a, (_entity_num)       ;; [4]  A = Number of entities
    ret                         ;; [3]


man_entity_getArrayHL::
    ld  de, #_entity_array      ;; [4] IX = Pointer to the entitiy array
    ret                         ;; [3]

;; Testing, ix += sizeof_e
man_entity_next_array_element::
    ld bc, #sizeof_e
    add ix, bc
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man Entity::Init
;; -------------------------------------------------------------
;;   Initializes the entity manager. It 
;;   set ups everything with 0 entities and
;;   ready to start creating new ones
;; INPUT: -
;; DESTROYS: AF, HL
;; RETURNS:
;;     HL: Pointer to the start of the entity array
;;
man_entity_init::
    ;; Init other subsidiary managers
    call    man_components_init

    ;; Reset all component vector values
    xor     a                           ;; [1] / _entity_num = 0
    ld      (_entity_num), a            ;; [4] \ Set number of entities to 0

    ld     hl, #_entity_array           ;; [3] / entity_pend = &_entities[0]
    ld     (_entity_pend), hl           ;; [5] \ Entities end pointer points to beginning of array

    ;; First entity must be marked as no entity
    ;; as it is the last valid entity in t  he array
    ex      de, hl                      ;; DE = HL
    ld      hl, #e_w                    ;; HL = Offset of e_w in entity parameters
    add     hl, de                      ;; HL = Entity[0] w_e memory address
    ld      (hl), #e_w_invalidEntity    ;; Entity[0] = invalid width
    ex      de, hl                      ;; HL = Pointer to the start of the entity array

    ret                                 ;; [3] Everything initialized. Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man Entity::New
;; -------------------------------------------------------------
;;   Adds a new entity component to the array
;;   without initializing it. It does not 
;;   perform any check for space in the array.
;; DESTROYS: F, BC, DE, HL
;; RETURN:
;;     DE: Points to added element
;;     BC: Sizeof (Entity_t)
;;     HL: Pointer to the last entity.
;; 
man_entity_new::
    ;; Increment number of reserved entities
    ld      hl, #_entity_num        ;; [3] / entity_num++
    inc     (hl)                    ;; [2] \

    ;; Increment Array end pointer to point to the next
    ;; free element in the array
    ld      hl, (_entity_pend)      ;; / [5]
    ld       d, h                   ;; | [1] / DE = HL
    ld       e, l                   ;; | [1] \ DE Points to the added element
    ld      bc, #sizeof_e           ;; | [3] entity_pend += sizeof(Entity_t)
    add     hl, bc                  ;; | [3]
    ld      (_entity_pend), hl      ;; \ [5] Store pend pointer update

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man Entity::Create
;; -------------------------------------------------------------
;;   Creates and initialized new entity.
;;   Returns a pointer to newly created entity in IX
;; INPUT: 
;;     HL : Pointer to initializer values for the entity to be created
;; DESTROYS: F, BC, HL, DE
;; STACK USE: 2 bytes
;; RETURN:
;;     IX: Pointer to the beginning of the created entity
;;
man_entity_create::
    push    hl                      ;; [4] Save pointer to the initialization values
    call    man_entity_new          ;; [x] Add a new entity to the vector first available position

    ;; IX = DE (IX returns the value of the pointer to the entity created)
    ld__ixh_d                       ;; [2] / IX = DE
    ld__ixl_e                       ;; [2] \ IX Points to the new entity space

    ;; Copy initialization values to the new entity
    ;; DE points to the new added entity
    ;; BC holds sizeof(Entity_t)
    ;; Copy initialization values
    pop     hl                      ;; [3] HL = Pointer to the initialization values
    ldir                            ;; [x] Initialization values -> new entity

    ;; Check for components to add the entity to
    ;; its correspondent pointer arrays in other managers
    ld      a, e_cmp_type(ix)       ;; [5] A = Entity_t.component_type (bit array)
    and     #e_cmp_AI               ;; [2] / If (Has AI component)
    jr      z, _noAI
_AI:
    ld      a, #e_cmpID_AI          ;; [2]
    call    man_components_add      ;; [x] Add entity to its correspondent manager
_noAI:
    ld      a, e_cmp_type(ix)
    and     #e_cmp_Physics
    jr      z, _noPhysics
_Physics:
    ld      a, #e_cmpID_Physics
    call    man_components_add
_noPhysics:
    ld      a, e_cmp_type(ix)
    and     #e_cmp_Animation
    jr      z, _noAnimation
_Animation:
    ld      a, #e_cmpID_Animation
    call    man_components_add
_noAnimation:


    ret                             ;; [3]


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man Entity::Erase Entity
;; -------------------------------------------------------------
;;   Deletes an entity and updates array values.
;;   If removed element is not the last one, move the last
;;   one to the freed position.
;; INPUT: 
;;     IX : Pointer to element to delete
;; DESTROYS: AF, BC, DE, HL 
;;
man_entity_erase_entity::
    ;; Decrement number of reserved entities
    ld      hl, #_entity_num        ;; [3] / entity_num--
    dec     (hl)                    ;; [2] \

    ;; Decrement pend one position
    ld      hl, (_entity_pend)      ;; / [5]
    ld      bc, #-sizeof_e          ;; | [3] entity_pend -= sizeof(Entity_t)
    add     hl, bc                  ;; | [3]
    ld      (_entity_pend), hl      ;; \ [5] Store pend pointer update

    ;; Check if pend points to deleted element, which means
    ;; we're deleting the last element
    ;{
        ;; IX -> Points to element to delete
        ;; HL -> Points to last element
        check_low_address:
            ld  a, l
            cp__ixl
            jr  z, check_high_address        ;; Low address equal, check high address
            jr  not_last_element_deleted

        check_high_address:
            ld  a, h
            cp__ixh
            jr  nz, not_last_element_deleted ;; High address not equal
            
            ;; High address equal => deleting last element
            last_element_deleted:
                ;; Set Entity.width to invalid
                ld      e_w(ix), #e_w_invalidEntity
                ret  z
    ;}

    ;; Copy last element to freed position, overwriting it
    not_last_element_deleted:
        push    hl                  ;; Save last entity

        ;; Alternative: push ix, pop de
        ld__d_ixh                   ;; DE = IX = Entity to erase
        ld__e_ixl

        ld  bc, #sizeof_e           ;; [3]
        ldir                        ;; Copy last element to freed element, overwriting it 
                                    ;; (Copy HL into DE, BC times)
                                    
        pop     ix                  ;; Load last entity so it can be marked as non valid

        ;; Set last Entity.width to invalid
        ld      e_w(ix), #e_w_invalidEntity

    ret                         ;; [3]