;;
;; GAME SYSTEMS MANAGER
;;    Manages all the systems for a game instance

.include "cpctelera.h.s"
.include "cmp/entity.h.s"
.include "sys/physics.h.s"
.include "sys/render.h.s"
.include "sys/input.h.s"
.include "sys/ai_control.h.s"
.include "sys/animation.h.s"
.include "sys/game_status.h.s"
.include "sys/level.h.s"
.include "man/level.h.s"
.include "man/interface.h.s"
.include "man/entity.h.s"
.include "man/menu.h.s"
.include "man/patrol.h.s"
.include "man/animation.h.s"
.include "assets/assets.h.s"

.module game_manager

;; ==================================================================
;; Manager Configuration Constants

;; ==================================================================
;; Manager Member Variables
hud_height  = 40
hero_x      = 4
hero_y      = 199-hud_height-16  
e_cmp_ai_entity          = e_cmp_default | e_cmp_AI
e_cmp_ai_entity_animated = e_cmp_ai_entity | e_cmp_Animation
e_cmp_animated           = e_cmp_default | e_cmp_Animation
hero: DefineCmp_Entity  hero_x, hero_y,   0,  0,    6,    16, e_cmp_animated, _sp_mainChar_00, _player_walking_right,   e_ai_st_noAI
default: DefineCmp_Entity   0,    0,   0,  0,    6,    12, e_cmp_ai_entity,    _sp_enemy,         nullptr,   e_ai_st_stand_by

;ent1: DefineCmp_Entity   0,    0,   0,  0,    6,    16, e_cmp_animated,   _sp_mainChar_00, _player_standing_right,   e_ai_st_noAI
;ent2: DefineCmp_Entity  70,   40,   0,  0,    6,    12, e_cmp_ai_entity,        _sp_enemy,         nullptr,   e_ai_st_stand_by
;ent3: DefineCmp_Entity  40,  120,   0,  0,    6,    12, e_cmp_ai_entity,        _sp_enemy,         nullptr,   e_ai_st_stand_by


;; ==================================================================
;; Manager Public Functions

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man Game: Init
;;   Initializes the Game Manager to set
;; it up for the start of a new game.
;; INPUT: -
;; DESTROYS: AF, BC, DE, HL, IX
;;
man_game_init::

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MENU MANAGEMENT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Init Main Menu Manager
    call    man_mainMenu_init

    ;; Wait until player starts the game
    call    man_mainMenu_update
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GAME STARTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_game_starts:
    ;; Init Entity Manager
    call    man_entity_init         ;; HL = Entity array
    call    man_patrol_init         ;; PATROL

    ;; Init systems
    call    sys_physics_init        ;; Physics
    call    sys_input_init          ;; Input
    call    sys_eren_init           ;; Rendering

    call    sys_ai_control_init     ;; Destroys HL!

    ;; Draws level 
    call    sys_lvl_init         ;; Level rendering

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GAME STATUS 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    call    sys_game_status_init


    ;; Render level
    call    sys_lvl_render_level

    ;; Animation system
    call    sys_animation_init

    ;;  HUD System
    call    man_interface_init

    ;; Spawn the main character
    ld      hl, #hero
    call    man_entity_create
    
    
    ;;; Spawn enemies depending on the current level to
    ;;; to play
    ld      iy, #default
    call    man_level_get_current_level_data    ;; hl = Current level data
    inc     hl
    inc     hl
    inc     hl
    inc     hl                                  ;; hl points to the enemies to fight
    
    _looper:
    ;; Get pointer to next enemy in DE
    ld       e, (hl)          ;; [2] /      E == X
    inc     hl                ;; [2] |
    ld       d, (hl)          ;; [2] \      D == Y
    inc     hl                ;; [2] HL += 2 (Points patrol)

    ;; Check for next enemy position being nullptr
    ;; in which case return
    ld       a, e             ;; [1]   /
    or       d                ;; [1]   | IF (DE == nullptr)
    ret      z                ;; [3/4] \ THEN Return
    
    ;; Modifiy position of default entity
    ld      e_x(iy), e          
    ld      e_y(iy), d

    ;; Assign patrol
    ld       e, (hl)          ;; [2] /      E == patrol step low
    inc     hl                ;; [2] |
    ld       d, (hl)          ;; [2] \      D == patrol step high
    inc     hl                ;; [2] HL += 2 (Points to next entity)

    ld      e_ai_patrol_step_l(iy), e          
    ld      e_ai_patrol_step_h(iy), d     
    
    push    hl                  ;; Save HL for next iteration
    ;; Create new entity with default values but corresponding
    ;; position in map
    ld      hl, #default
    call    man_entity_create
    pop     hl                  ;; Retrieve HL
    jr      _looper
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man Game: Update
;;   Updates the game cycle doing everything except
;; the rendering. Rendering is considered apart for 
;; its time constraints
;; INPUT: -
;; DESTROYS: -
;;
man_game_update::

    ;; CHECK FOR INPUT
    call    man_entity_getArray
    call    sys_input_update

    ;; UPDATE AI
    call    man_entity_getArray_IY
    call    sys_ai_control_update   ;; AI
    
    ;; Collisions
    call    man_entity_getArray
    call    man_entity_getArray_IY
    dec     a                   ;; Necessary because player and enemies
    ld      de, #sizeof_e       ;; are still grouped in the same array
    add     iy, de              ;; So IY must start in the second position, with one element less
    call    sys_physics_check_player_collision
    ;; UPDATE PHYSICS
    call    man_entity_getArray
    call    sys_physics_update

;;;;;;;;;;;; ANIMATION ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    call    sys_animation_update
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;; UPDATE GAME STATUS
    call    man_entity_getArray
    call    sys_game_status_update


    ;; CHECK IF DEFEAT
    call    sys_game_status_getStatus
    cp      #global_status_defeat
    jr      z,man_defeat


    ;; CHECK IF VICTORY - TODO
    call    sys_game_status_getStatus
    cp      #global_status_victory
    jr      z,man_victory

    ret

man_game_delete::

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GAME ENDS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;call    man_entity_delete       ;; TODO
    call    man_patrol_delete         ;; TODO

    push    hl                      ;; AI
    call    sys_ai_control_delete       ;; TODO
    pop     hl
    
    call    sys_physics_delete        ;; TODO
    call    sys_input_delete          ;; TODO
    call    sys_eren_delete           ;; TODO
    ;; Draws level 
    call    sys_lvl_delete         ;; TODO

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GAME STATUS 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    call    sys_game_status_delete      ;; TODO

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MENU MANAGEMENT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;; Init Main Menu Manager
    call    man_mainMenu_delete         ;; TODO


    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man Game: Render
;;   Does rendering process apart from
;; game update to account for its time constraints
;; INPUT: -
;; DESTROYS: -
;;
man_game_render::
    call    man_entity_getArray ;; TODO, no deberia hacer falta
    call    sys_eren_update
    
    ret

man_defeat:
    call man_defeatMenu

    ;; Restart game
    ;; TODO, reiniciar el nivel
    jp man_game_terminate

man_victory:
    call man_victoryMenu

    ;; Restart game
    ;; TODO, cargar el siguiente nivel o pantalla final
    jp man_game_terminate

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Man Game: Terminate
;;   Ends a game instance, and resets it
;; INPUT: -
;; DESTROYS: -
;;
man_game_terminate:
    call    sys_eren_terminate  ;; Reset first render function

    jp _game_starts