;;
;;  ANIMATION SYSTEM
;;
.include "cpctelera.h.s"
.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "man/animation.h.s"
.include "man/components.h.s"
.include "assets/assets.h.s"

.module sys_animation

;; ==========================================================================
;; PUBLIC FUNCTIONS
;; ==========================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ANIMATION::Init
;; ---------------------------------------------------------------------
;;    Initializes the Animation system
;;  INPUT: -
;;  DESTROYS: HL
;; 
sys_animation_init::
  ;; Get a pointer to the Animation_Array_Ptr
  ;; and save it for future use
  ld  a, #e_cmpID_Animation       ;; [2]
  call  man_components_getArrayHL ;; [x]
  ld  (_ent_array_ptr), hl        ;; [5] Store Entity_Animation_Ptr[] for later use
  ret                             ;; [3] Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ANIMATION::Update
;; ---------------------------------------------------------------------
;;    Updates all animated entities with their next
;;    animation sprite
;;  INPUT: -
;;  DESTROYS: AF, DE, HL, IX
;;
sys_animation_update::
  _ent_array_ptr = .+1
  ld      hl, #0x0000       ;; [3] HL = &Animation_Entity_Array_Ptr[0] (Array of pointers)

_loop:
  ;; Get pointer to next Animation-Entity in DE
  ld       e, (hl)          ;; [2] /
  inc     hl                ;; [2] | DE = &Animation_Entity 
  ld       d, (hl)          ;; [2] \
  inc     hl                ;; [2] HL += 2 (Points to next pointer)
  
  ;; Check for &Animation_Entity being nullptr
  ;; in which case return
  ld       a, e             ;; [1]   /
  or       d                ;; [1]   | IF (DE == nullptr)
  ret      z                ;; [3/4] \ THEN Return

  ;;  IX = DE (IX = &Animation_Entity)
  ld__ixl_e                 ;; [2] / IX = DE = &Animation_Entity
  ld__ixh_d                 ;; [2] \ 

  call  calculateAnimStatus ;; [x] B = Anim Status

  ;; Check if animation status is the same as before
  ld  a, e_anim_st(ix)      ;; A = Current Entity Anim Status
  cp  b                     ;; If ( A == B )
  jr  z, _continueAnim      ;; THEN Continue Animation 

_newAnimationStatus:
  ld  e_anim_st(ix), b      ;; Store new Animation Status

  ex  de, hl                ;; Save HL into DE

  call  man_animation_getAnimHL  ;; [x] HL Points to Animation Pointer

  ld  e_anim_ptr_l(ix), l   ;; / Store next animation step
  ld  e_anim_ptr_h(ix), h   ;; \ into the entity

  ex  de, hl                ;; Retrieve HL

_continueAnim:
  ld  e, e_anim_ptr_l(ix)   ;; / DE = Current step in animation
  ld  d, e_anim_ptr_h(ix)   ;; \

  ex  de, hl                ;; Save HL into DE, to use it with Current animation step

_drawNextAnimationStep:
  ld  c, (hl)               ;; /
  inc hl                    ;; | BC = Current sprite
  ld  b, (hl)               ;; \
  inc hl                    ;; HL += 2 (Points to step frames remaining)

  ld  a, c                  ;; / IF { Current Sprite (BC) == nullptr }
  or  b                     ;; | THEN
  jr  z, _resetAnim         ;; \ reset Animation

  ld  e_pspr_l(ix), c       ;; / Store current sprite pointer
  ld  e_pspr_h(ix), b       ;; \ into the entity
    
  dec (hl)                  ;; Step frames--
  jr  z, _stepFinished      ;; IF (frames == 0)
                            ;; THEN current animation step finished
  _stepNotFinished:    
    jr _update_end
  _stepFinished:
    ;; Set frames back to its initial value
    inc hl                    ;; HL points to step frames initial value
    ld  a, (hl)               ;; A = Initial step frames
    dec hl                    ;; HL points to step frames remaining
    ld  (hl), a               ;; Reset step frames

    ;; Update animation pointer
    inc hl                    ;; /
    inc hl                    ;; \ HL Points to next animation step
    ld  e_anim_ptr_l(ix), l   ;; / Store next animation step
    ld  e_anim_ptr_h(ix), h   ;; \ into the entity

  _update_end:

    ex  de, hl                ;; Recover stored HL from DE (HL points to next entity)

    jr  _loop                 ;; Continue with next animation step

_resetAnim:
  ld  a, (hl)               ;; /
  inc hl                    ;; | HL = (*HL)
  ld  h, (hl)               ;; | HL Points to the first step
  ld  l,  a                 ;; \ in the animation

  jr  _drawNextAnimationStep  ;; Draw first animation step

;; ==========================================================================
;; PRIVATE FUNCTIONS
;; ==========================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_ANIMATION::calculateAnimStatus
;; ---------------------------------------------------------------------
;;    Calculates next animation status 
;;  according to velocities
;;  INPUT:
;;    IX: Entity_t 
;;  DESTROYS: AF
;;  OUTPUT:
;;    B: Anim Status
calculateAnimStatus:
  ld  b, #0           ;; B = Original status

  ;; Check Entity Orientation
  ld  a, e_orientation(ix)
  cp  #e_orientation_left
  jr  z, _looking_left

_looking_right:
  set 0, b            ;; Activate 1st bit of B (XXXX XXX1) 
                      ;; 1 = Looking Right
_looking_left:
  ;; Check if Entity is attacking
  ld      a, e_meleeattackstate(ix) ;; A == e.meleeAttackState
  cp      #melee_notattacking       ;; IF ( A == not_Attacking)
  jr      z, _not_attacking         ;; THEN Entity isn't attacking

_attacking:
  set 4, b            ;; B = 0001 000X
  ret                 ;; Attacking animation doesn't need more bits

_not_attacking:
  ;; Check if Entity is moving on X axis
  ld  a, e_vx(ix)     ;; A = Entity.vx
  or  a               ;; IF ( A == 0 )
  jr  z, _vxZero      ;; THEN vx = 0

  set 1, b            ;; Moving on X axis

_vxZero:
  ;; Check if Entity is moving on Y axis
  ld  a, e_vy(ix)     ;; A = Entity.vy
  or  a               ;; IF ( A == 0 )
  jr  z, _vyZero      ;; THEN vy = 0

  set 3, b            ;; Moving on Y axis

  ;; Check if Entity is falling or jumping
_vyNoZero:
  jp  m, _vyNegative  ;; Flag S activated (negative speed)
_vyPositive:
  set 2, b            ;; Entity is jumping 
  jr  _vyZero

_vyNegative:
_vyZero:
  ret


checkAnimFramesLeft:
