;;
;;  ENTITY PHYSICS MOVEMENT
;;    Moves entities according to their velocity
;;    Takes int account screen borders
;;
.include "cpctelera.h.s"
.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "sys/level.h.s"
.include "man/level.h.s"
.include "assets/assets.h.s"

.module sys_entity_physics

;;==========================================================================
;; Physics system constants
screen_width    =  80       ;; (320 / 4 pixels)
screen_height   = 168       ;; (200-HUD size pixels)

gravity_force   =   1       ;; Move 1 byte on Y axis 

screen_width_pixel_to_tile_ratio  = 4
screen_height_pixel_to_tile_ratio = 8

test_bit                = 4
maximum_fall_velocity   = 8

_current_level:  .dw     0x0000
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS::Init
;; ---------------------------------------------------------------------
;;    Updates all physic components of entities by checking boundies.
;;    Assumes that entities are contiguous and all valid.
;;    Assumes theres is at least one entity in the array.
;;  INPUT:
;;      HL = Pointer to the entity array
;;
sys_physics_init::
  ld  (_ent_array_ptr_ix), hl    ;; [6]

  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS::Delete
sys_physics_delete::
    ret    

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS::applyGravity
;;   Applies gravity to the indicated entity
;;   PC and NPCs. 
;;   Checks if character is jumping or is on
;;   ground, otherwise applies gravity
;;  INPUT:
;;     IX = Pointer to the entity 
;;  DESTROYS: AF
;;
sys_physics_applyGravity::
    ;; Check if entity is jumping
    ld      a, e_jumpstate(ix)         ;; [2] A = -1
    cp      #jumpstate_nojump           ;; [X] Check if the entity isn't jumping
    ret     nz                          ;; [2/3] Return if entity IS jumping

    ;; TODO, check if entity is on ground

    ;; If entity is not jumping nor on ground, apply gravity

    ;; Apply gravity by adding gravity_force to e_vy
    ld      a, e_vy(ix)             ;; A = Entity.vy
    cp      #maximum_fall_velocity
    ret     z

    add     #gravity_force          ;; A += gravity_force
    ld      e_vy(ix), a             ;; Entity.vy = A

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS::update
;; ---------------------------------------------------------------------
;;    Updates all physic components of entities by checking boundies.
;;    Assumes that entities are contiguous and all valid.
;;    Assumes theres is at least one entity in the array.
;;  INPUT:
;;      HL = Pointer to the entity array
;;  DESTROYS: AF, BC, DE, IX
;;  STACK USE: 2 bytes
;;
sys_physics_update::
    _ent_array_ptr_ix = .+2
    ld  ix, #0x0000             ;; [4]

_update_loop:
    ;; Check if valid element
    ld  a, e_w(ix)
    cp  #e_w_invalidEntity
    ret z

    ;; Apply gravity
    call    sys_physics_applyGravity

    ;; Check brackground collision for entity
    call    sys_physics_check_background_collision

    ;; UPDATE X
    ld       a, #screen_width + 1   ;; [2] /
    sub     e_w(ix)                 ;; [5] | C = Screen width + 1 - entity_width
    ld       c, a                   ;; [1] \ 

    ld       a, e_x(ix)             ;; [6] A = Entity.x
    add     e_vx(ix)                ;; [5] Entity.x += Entity.vx
    cp       c                      ;; [5] Check for entity x > Screen width 
    jr      nc, invalid_x           ;; [2/3] If yes, then undo movement
valid_x:    
        ld      e_x(ix), a          ;; [6] New X is valid. Store in entity updated value
        jr      endif_x             ;; [3] Endif
invalid_x:
        ;ld      a, e_vx(ix)         ;; [6] / 
        ;neg                         ;; [2] | Entity.vx = - Entity.vx
        ;ld      e_vx(ix), a         ;; [6] \
endif_x:

    ;; UPDATE Y
    ld       a, #screen_height + 1      ;; [2]
    sub     e_h(ix)                     ;; [5] C = Screen height + 1 - entity height
    ld       c, a                       ;; [1]

    ld       a, e_y(ix)                 ;; [6] A = Entity.y
    add     e_vy(ix)                    ;; [5] Entity.y += Entity.vy
    cp       c                          ;; [1] Check for entity.y > Screen height + 1 - entity height
    jr      nc, invalid_y               ;; [2/3] If yes, then undo the movement
valid_y:
        ld      e_y(ix), a              ;; [6] Store result
        jr      endif_y                 ;; [3] Endif
invalid_y:
        ;ld      a, e_vy(ix)             ;; [6] /
        ;neg                             ;; [2] | Entity.vy = - entity.vy
        ;ld      e_vy(ix), a             ;; [6] \
endif_y:

    ld      de, #sizeof_e               ;; DE = sizeof(Entity_t)
    add     ix, de                      ;; IX Points to next entity_t in array
    jr      _update_loop                ;; Continue updating


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS::Check_Player_collision
;; ---------------------------------------------------------------------
;;    Checks a collision between the main character and the rest of
;;  entities.
;;    It assumes there are at least two valid entities.
;;  INPUT:
;;      IX = Pointer to the player
;;      IY = Pointer to the entity array
;;      A  = Number of elements in the array
;;  DESTROYS: 
;;      AF, BC, DE, HL, IX, IY
;;  STACK USE:
;;  RETURN:
;;
sys_physics_check_player_collision::

;; Check if player is on a timeout
check_timeout:
    ld      c, a                        ;; Store value temporary
   ld       a, e_colision_timeout(ix)
   cp       #colision_no_timeout_colision
   jp       z, _collision_loop_init     ;; No timeout, chec entities

   ;; IS IN TIMEOUT, DECREMENT TIMEOUT AND EXIT
   ld       a, e_colision_timeout(ix)
   dec     a
   ld      e_colision_timeout(ix),a
   ret

    ;; Restore value of NUmEntities
_collision_loop_init:
    ld      a,c
    or      a
    ret     z       ;; No entities so no need for check

    ;; There are elements, lets check them
    ld      a,c

_collision_loop:
    push    af       ;; Store a

    call    sys_physics_check_collision     ;; Get collision status

    ld       a, h                           ;; A == 1 collision with enemy
    cp      #1
    jp      z, cpc_collision
                           
    cp      #2                              ;; A == 2 collision with melee attack
    jp      z, cpc_melee_collision

    jr      cpc_nocollision                 ;; Else no collision
cpc_collision:
    ;; Had collision, so player looses 1 life
    ld      a, #e_st_lostlife
    ld      e_status(ix),a

    jr cpc_nocollision

cpc_melee_collision:
    ;; Enemy was hit by melee attack, so is dead
    ld      a,#e_st_dead
    ld      e_status(iy),a

    jr cpc_nocollision

cpc_nocollision:
    pop     af                              ;; [1] A = number of entities left
    dec      a                              ;; [1] entities --
    ret      z                              ;; [3/4] If no entities left, return

    ld      de, #sizeof_e                   ;; [3] DE = sizeof entity
    add     iy, de                          ;; [2] IY Points to the next entity in array
    jr  _collision_loop                     ;; [2] Continue updating

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS::component_CheckCollision
;; ---------------------------------------------------
;;  Checks collision between two components on axis X AND Y.
;; It assumes both components are valid.
;; INPUT:
;;      IX = Pointer to first component   ;; 
;;      IY = Pointer to second component  ;;    
;; RETURNS: 
;;      H  =  
;;          0 == no collision
;;          1 == collision entity-entity
;;          2 == collision entity melee attack -entity        
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_check_collision:  
    
check_colision:
    ;; Check collision on X axis
    ld       a, e_x(ix)                      ;; A = X position of the first component
    ld       b, e_x(iy)                      ;; B = X position of the second component
    ld       c, e_w(ix)                      ;; C = width of the first component
    ld       d, e_w(iy)                      ;; D = width of the second component
    call    sys_physics_check_collision_on_axis
    dec      h
    jp      nz, no_collision_on_x

collision_on_x:
        ;; Check collision on Y axis
        ld     a, e_y(ix)                  ;; A = Y position of the first component
        ld     b, e_y(iy)                  ;; B = Y position of the second component
        ld     c, e_h(ix)                  ;; C = Height of the first component
        ld     d, e_h(iy)                  ;; D = Height of the second component
        call  sys_physics_check_collision_on_axis

no_collision_on_x:

    ;; CHECK IF ATTACKING
    ld      a, e_meleeattackstate(ix)
    cp      #melee_notattacking
    jp      z, no_collision_on_x_melee

melee_attack_collision:
    ;; attack X -- entity x + entity width
    ld       c, e_x(ix)             ;; A = X of entity 1

    ld      a, e_orientation(ix)
    and     a
    jr      z, _attacking_left


_attacking_right:
    ld       a, e_w(ix)                ;; a = width
    add      c
    jp      _calculate_attack_position
_attacking_left:
    ld      a,e_x(ix)
    ld      c, e_melee_attack_width(ix)
    sub     c    
_calculate_attack_position:

    ld       b, e_x(iy)                     ;; B = X position of second entity
    ld       c, e_melee_attack_width(ix)    ;; C = Width of attack
    ld       d, e_w(iy)                     ;; D = Width of entity 2

    call    sys_physics_check_collision_on_axis  ;; CHECK X COLLISION
    dec      h  
    jp      nz, no_collision_on_x_melee          

    collision_on_x_melee:

        ld  b, e_y(iy)                      
        ld  a, e_y(ix)                       ;; A = Y of melee attack
        add     a,#6
        ld  c, e_melee_attack_height(iy)     ;; C = Height of melee attack
        ld  d, e_h(iy)                       ;; D = Height of entity 2
        call  sys_physics_check_collision_on_axis
        ld  a,h
        or  a
        jr  z, no_collision_on_x_melee

        ;; IF NO JUMP, MELEE COLLISION IS TRUE
        ld  h, #2

no_collision_on_x_melee:

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS::component_Check_BackGround_Collision
;; ---------------------------------------------------
;;  Checks collision between a component and the background
;; INPUT:
;;      IX = Pointer to first component   
;;      DE = Pointer to second background tilemap     
;; RETURNS: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_check_background_collision::
    ;ld  hl, #_lvl0
    call    man_level_get_current_level
    ld  (_current_level), de

    call    sys_physics_retrieve_ur_sidetile
    push    hl
    call    sys_physics_retrieve_ur_tile
    push    hl

    call    sys_physics_retrieve_ul_sidetile
    push    hl
    call    sys_physics_retrieve_ul_tile
    push    hl
    
    call    sys_physics_retrieve_lr_sidetile
    push    hl
    call    sys_physics_retrieve_lr_tile
    push    hl
    
    call    sys_physics_retrieve_ll_sidetile
    push    hl
    call    sys_physics_retrieve_ll_tile
    push    hl
    
    ;; Checl lower left tile
    ld      hl, #_current_level
    ld       e, (hl)
    inc     hl
    ld       d, (hl)
    pop     hl
    add     hl, de                          ;; Tile memory address

    ld       a, (hl)                        ;; A = Tile value
    ;or       a                              ;; Check if a == 0
    bit      #test_bit, a
    jp       z, #_no_ground_left           ;; If A == 0, tile is not collidible
    
    ;; We are touching ground, update the value
    ld      e_gr_st(ix), #e_gr_st_ground

    ;; If Y speed is lower than zero, set it to zero,
    ld       a, e_vy(ix)
    or       a
    jp       m, #_ll_tile_left
        ld      e_vy(ix), #0                    ;; Set speed == 0
        jr   #_ll_tile_left

_no_ground_left:
    ;; Lower tile isn't touching ground, update the value
    ld      e_gr_st(ix), #e_gr_st_not_ground

_ll_tile_left:
    
    ;; Check left tile on the lower right
    ld      hl, #_current_level
    ld       e, (hl)
    inc     hl
    ld       d, (hl)
    pop     hl
    add     hl, de

    ld       a, (hl)
     ;or       a                              ;; Check if a == 0
    bit      #test_bit, a
    jp       z, #_ll_tile_end

    ;; If moving to the left, keep moving
    ld       a, e_vx(ix)
    or       a
    jp       p, #_ll_tile_end
        ld      e_vx(ix), #0

_ll_tile_end:
    ;; Check lower right tile
    ld      hl, #_current_level
    ld       e, (hl)
    inc     hl
    ld       d, (hl)
    pop     hl
    add     hl, de                          ;; Tile memory address

    ld       a, (hl)                        ;; A = Tile value
     ;or       a                              ;; Check if a == 0
    bit      #test_bit, a
    jp       z, #_no_ground_right             ;; If A == 0, tile is not collidible

    ;; We are touching ground, update the value
    ld      e_gr_st(ix), #e_gr_st_ground

    ;; If Y speed is lower than zero, set it to zero,
    ld       a, e_vy(ix)
    or       a
    jp       m, #_lr_tile_right
        ld      e_vy(ix), #0                    ;; Set speed == 0
        jr      #_lr_tile_right

_no_ground_right:
        ;; Lower tile isn't touching ground, update the value
        ld      a, e_gr_st(ix)
        and     #e_gr_st_not_ground
        ld      e_gr_st(ix), a
        
_lr_tile_right:
    ld      hl, #_current_level
    ld       e, (hl)
    inc     hl
    ld       d, (hl)
    pop     hl
    add     hl, de

    ld       a, (hl)
    ;or       a                              ;; Check if a == 0
    bit      #test_bit, a
    jp       z, #_l_tiles_end

    ld       a, e_vx(ix)
    or       a
    jp       m, #_l_tiles_end
        ld      e_vx(ix), #0                    ;; Set speed == 0

_l_tiles_end:
    ;; Checl upper left tile
    ld      hl, #_current_level
    ld       e, (hl)
    inc     hl
    ld       d, (hl)
    pop     hl
    add     hl, de                          ;; Tile memory address

    ld       a, (hl)                        ;; A = Tile value
    ;or       a                              ;; Check if a == 0
    bit      #test_bit, a
    jp       z, #_ul_tile_left               ;; If A == 0, tile is not collidible

    ;; If Y speed is bigger than zero, set it to zero,
    ld       a, e_vy(ix)
    or       a
    jp       p, #_ul_tile_left
        ld      e_vy(ix), #0                    ;; Set speed == 0

_ul_tile_left:
    ;; Check left tile on the upper right
    ld      hl, #_current_level
    ld       e, (hl)
    inc     hl
    ld       d, (hl)
    pop     hl
    add     hl, de

    ld       a, (hl)
    ;or       a                              ;; Check if a == 0
    bit      #test_bit, a
    jp       z, #_ul_tile_end

    ;; If moving to the left, keep moving
    ld       a, e_vx(ix)
    or       a
    jp       p, #_ul_tile_end
        ld      e_vx(ix), #0

_ul_tile_end:

    ;; Check upper right tile
    ld      hl, #_current_level
    ld       e, (hl)
    inc     hl
    ld       d, (hl)
    pop     hl
    add     hl, de                                      ;; Tile memory address

    ld       a, (hl)                                    ;; A = Tile value
    ;or       a                              ;; Check if a == 0
    bit      #test_bit, a
    jp       z, #_ur_tile_right                         ;; If A == 0, tile is not collidible

    ;; If Y speed is bigger than zero, set it to zero,
    ld       a, e_vy(ix)
    or       a
    jp       p, #_ur_tile_right
        ld      e_vy(ix), #0                    ;; Set speed == 0

_ur_tile_right:
    ld      hl, #_current_level
    ld       e, (hl)
    inc     hl
    ld       d, (hl)
    pop     hl
    add     hl, de
    
    ld       a, (hl)
    ;or       a                              ;; Check if a == 0
    bit      #test_bit, a
    jp       z, #_u_tiles_end

    ld       a, e_vx(ix)
    or       a
    jp       m, #_u_tiles_end
        ld      e_vx(ix), #0                    ;; Set speed == 0

_u_tiles_end:
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_PHYSICS::check_collision_on_axis
;; ---------------------------------------------------
;;  Checks collision between two components on a determined axis.
;; The axis is defined by the elements passed on the A, B, C and D registers.
;; This function should receive, depending on which axis to check, the following
;; values:
;;      X axis:
;;          A & B = the X coordinate of the first and second components.
;;          C & D = the width of the first and second components.
;;      Y axis:
;;          A & B = The Y coordinate of the first and second components.
;;          C & D = the height of the first and second components.
;; INPUT: 
;;          A  = X/Y coordinate of first component
;;          B  = X/Y coordinate of second component
;;          C  = Width/Height of the first component
;;          D  = Width/Height of the second component
;; DESTROYS: 
;;          AF, BC, DE, HL
;; RETURNS: 
;;          H = 0 == no collision, 1 == collision         
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_check_collision_on_axis:
    ld    h, #0                      ;; [1] H = 0, default is no collision
    cp    b                          ;; [1] Compare to b, changes flags
    jp   m, collision_IX_Left        ;; [3] If a < b, IX component is upmost or leftmost of IY

    ;; IY_POS + IY_DIM - IX_POS <= 0
    ld    e, a                      ;; [1] E = IX_POS
    ld    a, b                      ;; [1] B = IY_POS of second component
    add   d                         ;; [1] D = IY_POS + IY_DIM
    dec   a                         ;; [1] A = IY_POS + IY_DIM - 1
    cp      e                       ;; [1] IY_POS + IY_DIM <= IX_POS
    jp   p, collision_yes           ;; [3] If IY_POS + IY_DIM > IX_POS, there is a collision
    jr   end_check_collision        ;; [2] if a < e, there is no collision in axis x,

collision_IX_Left:
    ;; IX_POS + IX_DIM - IY_POS <= 0
    add   c                         ;; [1] A = IX_POS + IX_DIM
    dec   a                         ;; [1] A = IX_POS + IX_DIM - 1
    cp   b                          ;; [1] IX_POS + IX_DIM <= IY_POS
    jp   m, end_check_collision     ;; [3] if IX_POS + IX_DIM < IY_POS, no collision
    ;;jr   z, end_check_collision     ;; [3] if IX_POS + IX_DIM = IY_POS, no collision
                                    ;; End if there is no collision, or continue 
                                    ;; and set H to 1 if there is.
collision_yes:
    ld    h, #1                     ;; Set H to 1 in case there is a collision.
end_check_collision:
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Performs the HL = (Y+height / 8) * 20 part of the calculus
;; common to all lower cornes of the sprites
;;  INPUT:
;;      IX = Entity
;;  OUTPUT:
;;      HL = (Y+height / 8) * 20
sys_physics_retrieve_l_t:
    ld       a, e_y(ix)                         ;; A = Y pos
    ld       c, e_h(ix)
    add      c                                  ;; A = Y pos + height
    dec      a                                  ;; A = Y pos + height - 1
    ld       c, e_vy(ix)                        ;; C = Y speed                     
    add      c 
    ld       c, a                               ;; C = Y + Height
    ld       d,#8                               ;; D = 8
    call    c_div_d                             ;; C = C / D
    
    ld       a, c                               ;; Y = Y / 8
    ld      de, #screen_tile_width              ;; DE = 20
    call    de_times_a                          ;; HL = (Y+height / 8) * 20
    
    ret

sys_physics_retrieve_l_t_sides:
    ld       a, e_y(ix)                         ;; A = Y pos
    ld       c, e_h(ix)
    add      c                                  ;; A = Y pos + height
    dec      a                                  ;; A = Y pos + height - 1
    ld       c, a                               ;; C = Y + Height
    ld       d,#8                               ;; D = 8
    call    c_div_d                             ;; C = C / D
    
    ld       a, c                               ;; Y = Y / 8
    ld      de, #screen_tile_width              ;; DE = 20
    call    de_times_a                          ;; HL = (Y+height / 8) * 20
    
    ret

sys_physics_retrieve_ll_tile::
    call    sys_physics_retrieve_l_t            ;; HL = (Y+height / 8) * 20

    ld       a, e_x(ix)                         ;; A = X pos
    ld       c, e_vx(ix)                        ;; C = X speed
    add      c                                  ;; A = X pos - X speed
    ld       c, a                               ;; C = A
    ld       d, #screen_width_pixel_to_tile_ratio     ;; D = 2
    call    c_div_d

    add     hl, bc                              ;; HL = (Y / 8) * 40 + (X / 2)

    ret

sys_physics_retrieve_ll_sidetile::
    call    sys_physics_retrieve_l_t_sides      ;; HL = (Y+height / 8) * 20

    ld       a, e_x(ix)                         ;; A = X pos
    ld       c, e_vx(ix)                        ;; C = X speed
    add      c                                  ;; A = X pos - X speed
    ld       c, a                               ;; C = A
    ld       d, #screen_width_pixel_to_tile_ratio     ;; D = 2
    call    c_div_d

    add     hl, bc                              ;; HL = (Y / 8) * 40 + (X / 2)

    ret

sys_physics_retrieve_lr_tile::
    call    sys_physics_retrieve_l_t            ;; HL = (Y+height / 8) * 20

    ld       a, e_x(ix)                         ;; A = X pos
    ld       c, e_w(ix)                         ;; C = Width
    add      c
    dec      a
    ld       c, e_vx(ix)                        ;; C = X speed
    add      c                                  ;; A = X pos + Width + X speed - 1
    ld       c, a
    ld       d, #screen_width_pixel_to_tile_ratio     ;; D = 2
    call    c_div_d

    add     hl, bc                              ;; HL = (Y / 8) * 40 + (X / 2)

    ret

sys_physics_retrieve_lr_sidetile::
    call    sys_physics_retrieve_l_t_sides      ;; HL = (Y+height / 8) * 20

    ld       a, e_x(ix)                         ;; A = X pos
    ld       c, e_w(ix)                         ;; C = Width
    add      c
    dec      a
    ld       c, e_vx(ix)                        ;; C = X speed
    add      c                                  ;; A = X pos + Width + X speed - 1
    ld       c, a
    ld       d, #screen_width_pixel_to_tile_ratio     ;; D = 2
    call    c_div_d

    add     hl, bc                              ;; HL = (Y / 8) * 40 + (X / 2)

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Performs the HL = (Y+height / 8) * 20 part of the calculus
;; common to all upper cornes of the sprites
;;  INPUT:
;;      IX = Entity
;;  OUTPUT:
;;      HL = (Y+height / 8) * 20
sys_physics_retrieve_u_t:
    ;; (Y / 8) * tilemap_width + (X / 2)
    ld       a, e_y(ix)                             ;; A = Y pos    
    ld       c, e_vy(ix)                            ;; C = Y speed                     
    add      c                                      ;; A = A + C
    ld       c, a
    ld       d, #screen_height_pixel_to_tile_ratio  ;; D = 8
    call    c_div_d                                 ;; C = C / D

    ld      a, c                                    ;; A =  Y/8
    ld      de, #screen_tile_width                  ;; DE = 40
    call    de_times_a                              ;; HL = (Y/8) * 40
    
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Performs the HL = (Y+height / 8) * 20 part of the calculus
;; common to all upper cornes of the sprites
;;  INPUT:
;;      IX = Entity
;;  OUTPUT:
;;      HL = (Y+height / 8) * 20
sys_physics_retrieve_u_t_sides:
    ;; (Y / 8) * tilemap_width + (X / 2)
    ld       c, e_y(ix)                             ;; A = Y pos    
    ld       d, #screen_height_pixel_to_tile_ratio  ;; D = 8
    call    c_div_d                                 ;; C = C / D

    ld      a, c                                    ;; A =  Y/8
    ld      de, #screen_tile_width                  ;; DE = 40
    call    de_times_a                              ;; HL = (Y/8) * 40
    
    ret

sys_physics_retrieve_ul_tile::
    ;; (Y / 8) * tilemap_width + (X / 2)
    call    sys_physics_retrieve_u_t                ;; HL = (Y/8) * 20

    ld       a, e_x(ix)                         ;; A = X pos
    ld       c, e_vx(ix)                        ;; C = X speed
    add      c                                  ;; A = X pos - X speed
    ld       c, a                               ;; C = A
    ld       d, #screen_width_pixel_to_tile_ratio     ;; D = 2
    call    c_div_d

    add     hl, bc 
    ret

sys_physics_retrieve_ul_sidetile::
    ;; (Y / 8) * tilemap_width + (X / 2)
    call    sys_physics_retrieve_u_t_sides      ;; HL = (Y/8) * 20

    ld       a, e_x(ix)                         ;; A = X pos
    ld       c, e_vx(ix)                        ;; C = X speed
    add      c                                  ;; A = X pos - X speed
    ld       c, a                               ;; C = A
    ld       d, #screen_width_pixel_to_tile_ratio     ;; D = 2
    call    c_div_d

    add     hl, bc 
    ret

sys_physics_retrieve_ur_tile::
    ;; (Y / 8) * tilemap_width + (X / 2)
    call    sys_physics_retrieve_u_t                ;; HL = (Y/8) * 20

    ld       a, e_x(ix)                         ;; A = X pos
    ld       c, e_w(ix)                         ;; C = Width
    add      c                                  ;; A = A + C
    dec      a                                  ;; A = X post + width - 1
    ld       c, e_vx(ix)                        ;; C = X speed
    add      c                                  ;; A = X pos + Width + X speed - 1
    ld       c, a
    ld       d, #screen_width_pixel_to_tile_ratio     ;; D = 2
    call    c_div_d

    ;;ld       b, #0                            ;; BC = X / 2
    add     hl, bc 
    ret

sys_physics_retrieve_ur_sidetile::
    ;; (Y / 8) * tilemap_width + (X / 2)
    call    sys_physics_retrieve_u_t_sides      ;; HL = (Y/8) * 20

    ld       a, e_x(ix)                         ;; A = X pos
    ld       c, e_w(ix)                         ;; C = Width
    add      c                                  ;; A = A + C
    dec      a                                  ;; A = X post + width - 1
    ld       c, e_vx(ix)                        ;; C = X speed
    add      c                                  ;; A = X pos + Width + X speed - 1
    ld       c, a
    ld       d, #screen_width_pixel_to_tile_ratio     ;; D = 2
    call    c_div_d

    ;;ld       b, #0                            ;; BC = X / 2
    add     hl, bc 
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:
;;      C = Numerator
;;      D = Denominator
;; OUTPUT:
;;      A = Remainder
;;      B = 0
;;      C = C / D 
c_div_d:
    ld       b, #8              ;; B = 8
    xor      a                  ;; A = 0

    sla      c                  ;; Shift C to the left
    rla                         ;; Rotate A to the left
    cp       d                  ;; If A < D stop iterating
    jr      c, #.+4
                                ;; ELSE
        inc      c              ;; C++
        sub      d              ;; D--
        djnz    #.-8      ;; B--, if B < 0, keep iterating
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  INPUT:
;;      DE = Factor
;;       A = Factor
;;  OUPUT:
;;       A = unchanged
;;       B = 0
;;      DE = unchanged
;;      HL = Product
de_times_a:
    ld       b, #8
    ld      hl, #0
        add     hl, hl      
        rlca    
        jr      nc, #.+3
            add     hl, de
        djnz    #.-5
    ret



